@@ -1,4 +1,4 @@
-﻿const URL_REGEX = /^(http:\/\/www\.|https:\/\/www\.|http:\/\/|https:\/\/)?[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/
+﻿const URL_REGEX = /((([A-Za-z]{3,9}:(?:\/\/)?)(?:[\-;:&=\+\$,\w]+@)?[A-Za-z0-9\.\-]+|(?:www\.|[\-;:&=\+\$,\w]+@)[A-Za-z0-9\.\-]+)((?:\/[\+~%\/\.\w\-_]*)?\??(?:[\-\+=&;%@\.\w_]*)#?(?:[\.\!\/\\\w]*))?)/
 const YOUTUBE_LINK_REGEX = /http(?:s?):\/\/(?:www\.)?youtu(?:be\.com\/watch\?v=|\.be\/)([\w\-\_]*)(&(amp;)?‌​[\w\?‌​=]*)?/gm
 
 function isUrl(url){
@@ -6,6 +6,19 @@ function isUrl(url){
     return (matches !== null && matches.length > 0)
 }
 
+function extractUrls(text) {
+    let lines = text.split("\n")
+    var urls = []
+
+    for (const line of lines) {
+        if (isUrl(line)) {
+            urls.push(line)
+        }
+    }
+
+    return urls
+}
+
 function isYoutubeLink(url) {
     let urls = url.match(YOUTUBE_LINK_REGEX)
 	
