@@ -17,6 +17,7 @@ class Settings(QObject):
     output_path_changed = Signal(str)
     file_format_changed = Signal(str)
     single_line_changed = Signal(bool)
+    theme_color_changed = Signal(str)
 
     def __init__(self, settings_path=None):
         super(Settings, self).__init__(None)
@@ -27,6 +28,7 @@ class Settings(QObject):
         self.output_path = QStandardPaths.writableLocation(QStandardPaths.DownloadLocation)
         self.file_format = "webm"
         self.single_line = True
+        self.theme_color = "#004d99"
 
         if os.path.isfile(Settings.CONFIG_PATH):
             self.load()
@@ -40,6 +42,7 @@ class Settings(QObject):
         self.output_path = settings.value("output_path", QStandardPaths.writableLocation(QStandardPaths.DownloadLocation))
         self.file_format = settings.value("file_format", "webm")
         self.single_line = bool(settings.value("single_line", "True") == "True")
+        self.theme_color = settings.value("theme_color", "#004d99")
         settings.endGroup()
 
     def save(self) -> None:
@@ -49,6 +52,7 @@ class Settings(QObject):
         settings.setValue("output_path", self.output_path)
         settings.setValue("file_format", self.file_format)
         settings.setValue("single_line", "True" if self.single_line else "False") # NOTE: QSettings has a problem with bool values(bool("False") = True)
+        settings.setValue("theme_color", self.theme_color)
         settings.endGroup()
 
     def read_input_link(self) -> str:
@@ -91,7 +95,19 @@ class Settings(QObject):
         self.single_line = new_single_line
         self.single_line_changed.emit(self.single_line)
 
+    def read_theme_color(self) -> str:
+        return self.theme_color
+
+    def set_theme_color(self, new_theme_color: str) -> None:
+        if self.theme_color == new_theme_color:
+            return
+
+        self.theme_color = new_theme_color
+        self.theme_color_changed.emit(self.theme_color)
+
+
     inputLink = Property(str, read_input_link, set_input_link, notify=input_link_changed)
     outputPath = Property(str, read_output_path, set_output_path, notify=output_path_changed)
     fileFormat = Property(str, read_file_format, set_file_format, notify=file_format_changed)
     singleLine = Property(bool, read_single_line, set_single_line, notify=single_line_changed)
+    themeColor = Property(str, read_theme_color, set_theme_color, notify=theme_color_changed)
